@@ -1,107 +1,97 @@
 package Apache::AuthCookie;
-{
-  $Apache::AuthCookie::VERSION = '3.19_01';
-}
-
-# ABSTRACT: Perl Authentication and Authorization via cookies
 
 use strict;
 
 use Carp;
 use mod_perl qw(1.07 StackedHandlers MethodHandlers Authen Authz);
-use Apache::Constants qw(:common M_GET FORBIDDEN OK REDIRECT);
-use Apache::AuthCookie::Params;
+use Apache::Constants qw(:common M_GET FORBIDDEN REDIRECT);
 use Apache::AuthCookie::Util;
-use Apache::AuthCookie::Autobox;
 use Apache::Util qw(escape_uri);
+use vars qw($VERSION);
+
+# $Id: AuthCookie.pm,v 1.3 2005/04/18 05:23:34 mschout Exp $
+$VERSION = '3.07';
 
 sub recognize_user ($$) {
-    my ($self, $r) = @_;
+  my ($self, $r) = @_;
+  my $debug = $r->dir_config("AuthCookieDebug") || 0;
+  my ($auth_type, $auth_name) = ($r->auth_type, $r->auth_name);
 
-    # only check if user is not already set
-    return DECLINED unless $r->connection->user->is_blank;
+  return DECLINED unless $auth_type && $auth_name;
 
-    my $debug = $r->dir_config("AuthCookieDebug") || 0;
-    my ($auth_type, $auth_name) = ($r->auth_type, $r->auth_name);
+  return DECLINED unless $r->header_in('Cookie');
 
-    return DECLINED if $auth_type->is_blank or $auth_name->is_blank;
+  my $cookie_name = $self->cookie_name($r);
 
-    return DECLINED if $r->header_in('Cookie')->is_blank;
+  my ($cookie) = $r->header_in('Cookie') =~ /$cookie_name=([^;]+)/;
+  $r->log_error("cookie $cookie_name is $cookie") if $debug >= 2;
+  return DECLINED unless $cookie;
 
-    my $cookie_name = $self->cookie_name($r);
+  my ($user,@args) = $auth_type->authen_ses_key($r, $cookie);
+  if ($user and scalar @args == 0) {
+    $r->log_error("user is $user") if $debug >= 2;
 
-    my ($cookie) = $r->header_in('Cookie') =~ /$cookie_name=([^;]+)/;
-    $r->log_error("cookie $cookie_name is $cookie") if $debug >= 2;
-    return DECLINED unless $cookie;
-
-    my ($user, @args) = $auth_type->authen_ses_key($r, $cookie);
-    if (!$user->is_blank and scalar @args == 0) {
-        $r->log_error("user is $user") if $debug >= 2;
-
-        # if SessionTimeout is on, send new cookie with new Expires.
-        if (my $expires = $r->dir_config("${auth_name}SessionTimeout")) {
-            $self->send_cookie($cookie, { expires => $expires });
-        }
-
-        $r->connection->user($user);
-    }
-    elsif (scalar @args > 0 and $auth_type->can('custom_errors')) {
-        return $auth_type->custom_errors($r, $user, @args);
+    # if SessionTimeout is on, send new cookie with new Expires.
+    if (my $expires = $r->dir_config("${auth_name}SessionTimeout")) {
+      $self->send_cookie($cookie, {expires => $expires});
     }
 
-    return $user->is_blank ? DECLINED : OK;
+    $r->connection->user($user);
+  } elsif (scalar @args > 0 and $auth_type->can('custom_errors')) {
+    return $auth_type->custom_errors($r, $user, @args);
+  }
+
+  return OK;
 }
 
 sub cookie_name {
-    my ($self, $r) = @_;
+  my ($self, $r) = @_;
 
-    my $auth_type = $r->auth_type;
-    my $auth_name = $r->auth_name;
+  my $auth_type = $r->auth_type;
+  my $auth_name = $r->auth_name;
 
-    my $cookie_name = $r->dir_config("${auth_name}CookieName")
-        || "${auth_type}_${auth_name}";
+  my $cookie_name = $r->dir_config("${auth_name}CookieName") ||
+                    "${auth_type}_${auth_name}";
 
-    return $cookie_name;
+  return $cookie_name;
 }
 
 sub handle_cache {
-    my $self = shift;
+  my $self = shift;
 
-    my $r = Apache->request;
+  my $r = Apache->request;
 
-    my $auth_name = $r->auth_name;
-    return unless $auth_name;
+  my $auth_name = $r->auth_name;
+  return unless $auth_name;
 
-    unless ($r->dir_config("${auth_name}Cache")) {
-        $r->no_cache(1);
-        $r->err_header_out(Pragma => 'no-cache');
-    }
+  unless ($r->dir_config("${auth_name}Cache")) {
+    $r->no_cache(1);
+    $r->err_header_out(Pragma => 'no-cache');
+  }
 }
 
 sub remove_cookie {
-    my $self = shift;
+  my $self = shift;
 
-    my $r = Apache->request;
+  my $r = Apache->request;
 
-    my $debug = $r->dir_config("AuthCookieDebug") || 0;
+  my $debug = $r->dir_config("AuthCookieDebug") || 0;
 
-    my $cookie_name = $self->cookie_name($r);
+  my $cookie_name = $self->cookie_name($r);
 
-    my $str = $self->cookie_string(
-        request => $r,
-        key     => $cookie_name,
-        value   => '',
-        expires => 'Mon, 21-May-1971 00:00:00 GMT'
-    );
+  my $str = $self->cookie_string( request => $r,
+                                  key     => $cookie_name,
+                                  value   => '',
+                                  expires => 'Mon, 21-May-1971 00:00:00 GMT' );
 
-    $r->err_headers_out->add("Set-Cookie" => "$str");
-
-    $r->log_error("removed cookie $cookie_name") if $debug >= 2;
+  $r->err_headers_out->add("Set-Cookie" => "$str");
+  $r->log_error("removed_cookie " . $r->err_headers_out->get("Set-Cookie"))
+    if $debug >= 2;
 }
 
 # convert current request to GET
 sub _convert_to_get {
-    my ($self, $r) = @_;
+    my ($self, $r, $args) = @_;
 
     return unless $r->method eq 'POST';
 
@@ -109,20 +99,15 @@ sub _convert_to_get {
 
     $r->log_error("Converting POST -> GET") if $debug >= 2;
 
-    my $args = $self->params($r);
+    my @pairs =();
+    while (my ($name, $value) = each %$args) {
+      # we dont want to copy login data, only extra data
+      next if $name eq 'destination'
+           or $name =~ /^credential_\d+$/;
 
-    my @pairs = ();
-
-    for my $name ($args->param) {
-        # we dont want to copy login data, only extra data
-        next if $name eq 'destination'
-             or $name =~ /^credential_\d+$/;
-
-        for my $v ($args->param($name)) {
-            push @pairs, escape_uri($name) . '=' . escape_uri($v);
-        }
+      $value = '' unless defined $value;
+      push @pairs, escape_uri($name) . '=' . escape_uri($value);
     }
-
     $r->args(join '&', @pairs) if scalar(@pairs) > 0;
 
     $r->method('GET');
@@ -130,412 +115,336 @@ sub _convert_to_get {
     $r->headers_in->unset('Content-Length');
 }
 
-sub params {
-    my ($self, $r) = @_;
-
-    return Apache::AuthCookie::Params->new($r);
-}
-
 sub login ($$) {
-    my ($self, $r) = @_;
-    my $debug = $r->dir_config("AuthCookieDebug") || 0;
+  my ($self, $r) = @_;
+  my $debug = $r->dir_config("AuthCookieDebug") || 0;
 
-    my ($auth_type, $auth_name) = ($r->auth_type, $r->auth_name);
+  my ($auth_type, $auth_name) = ($r->auth_type, $r->auth_name);
+  my %args = $r->method eq 'POST' ? $r->content : $r->args;
 
-    my $params = $self->params($r);
+  $self->_convert_to_get($r, \%args) if $r->method eq 'POST';
 
-    $self->_convert_to_get($r) if $r->method eq 'POST';
+  unless (exists $args{'destination'}) {
+    $r->log_error("No key 'destination' found in form data");
+    $r->subprocess_env('AuthCookieReason', 'no_cookie');
+    return $auth_type->login_form;
+  }
+  
+  # Get the credentials from the data posted by the client
+  my @credentials;
+  while (exists $args{"credential_" . ($#credentials + 1)}) {
+    $r->log_error("credential_" . ($#credentials + 1) . " " .
+		  $args{"credential_" . ($#credentials + 1)}) if ($debug >= 2);
+    push(@credentials, $args{"credential_" . ($#credentials + 1)});
+  }
+  
+  # Exchange the credentials for a session key.
+  my $ses_key = $self->authen_cred($r, @credentials);
+  unless ($ses_key) {
+    $r->log_error("Bad credentials") if $debug >=2;
+    $r->subprocess_env('AuthCookieReason', 'bad_credentials');
+    $r->uri($args{'destination'});
+    return $auth_type->login_form;
+  }
 
-    unless (defined $params->param('destination')) {
-        $r->log_error("No key 'destination' found in form data");
-        $r->subprocess_env('AuthCookieReason', 'no_cookie');
-        return $auth_type->login_form;
+  if ($debug >= 2) {
+    if (defined $ses_key) {
+      $r->log_error("ses_key $ses_key");
     }
-
-    # Get the credentials from the data posted by the client
-    my @credentials;
-    for (my $i = 0 ; defined $params->param("credential_$i") ; $i++) {
-        my $key = "credential_$i";
-        my $val = $params->param("credential_$i");
-        $r->log_error("$key $val") if $debug >= 2;
-        push @credentials, $val;
+    else {
+      $r->log_error("ses_key undefined");
     }
+  }
 
-    # save creds in pnotes in case login form script wants to use them.
-    $r->pnotes("${auth_name}Creds", \@credentials);
+  $self->send_cookie($ses_key);
 
-    # Exchange the credentials for a session key.
-    my $ses_key = $self->authen_cred($r, @credentials);
-    unless ($ses_key) {
-        $r->log_error("Bad credentials") if $debug >= 2;
-        $r->subprocess_env('AuthCookieReason', 'bad_credentials');
-        $r->uri($params->param('destination'));
-        return $auth_type->login_form;
-    }
+  $self->handle_cache;
 
-    if ($debug >= 2) {
-        if (defined $ses_key) {
-            $r->log_error("ses_key $ses_key");
-        }
-        else {
-            $r->log_error("ses_key undefined");
-        }
-    }
-
-    $self->send_cookie($ses_key);
-
-    $self->handle_cache;
-
-    $r->header_out(
-        "Location" => $self->untaint_destination($params->param('destination')));
-
-    return REDIRECT;
-}
-
-sub untaint_destination {
-    my ($self, $dest) = @_;
-
-    return Apache::AuthCookie::Util::escape_destination($dest);
+  $r->header_out("Location" => $args{'destination'});
+  return REDIRECT;
 }
 
 sub logout($$) {
-    my ($self, $r) = @_;
-    my $debug = $r->dir_config("AuthCookieDebug") || 0;
+  my ($self,$r) = @_;
+  my $debug = $r->dir_config("AuthCookieDebug") || 0;
+  
+  $self->remove_cookie;
 
-    $self->remove_cookie;
+  $self->handle_cache;
 
-    $self->handle_cache;
-
-    #my %args = $r->args;
-    #if (exists $args{'redirect'}) {
-    #  $r->err_header_out("Location" => $args{'redirect'});
-    #  return REDIRECT;
-    #} else {
-    #  $r->status(200);
-    #  return OK;
-    #}
+  #my %args = $r->args;
+  #if (exists $args{'redirect'}) {
+  #  $r->err_header_out("Location" => $args{'redirect'});
+  #  return REDIRECT;
+  #} else {
+  #  $r->status(200);
+  #  return OK;
+  #}
 }
 
 sub authenticate ($$) {
-    my ($auth_type, $r) = @_;
-    my $auth_user;
-    my $debug = $r->dir_config("AuthCookieDebug") || 0;
+  my ($auth_type, $r) = @_;
+  my $auth_user;
+  my $debug = $r->dir_config("AuthCookieDebug") || 0;
+  
+  $r->log_error("auth_type " . $auth_type) if ($debug >= 3);
+  return OK unless $r->is_initial_req; # Only authenticate the first internal request
+  
+  if ($r->auth_type ne $auth_type) {
+    # This location requires authentication because we are being called,
+    # but we don't handle this AuthType.
+    $r->log_error("AuthType mismatch: $auth_type =/= ".$r->auth_type) if $debug >= 3;
+    return DECLINED;
+  }
 
-    $r->log_error("auth_type " . $auth_type) if ($debug >= 3);
+  # Ok, the AuthType is $auth_type which we handle, what's the authentication
+  # realm's name?
+  my $auth_name = $r->auth_name;
+  $r->log_error("auth_name " . $auth_name) if $debug >= 2;
+  unless ($auth_name) {
+    $r->log_reason("AuthName not set, AuthType=$auth_type", $r->uri);
+    return SERVER_ERROR;
+  }
 
-    unless ($r->is_initial_req) {
-        if (defined $r->prev) {
-            # we are in a subrequest.  Just copy user from previous request.
-            $r->connection->user($r->prev->connection->user);
-        }
-        return OK;
+  # Get the Cookie header. If there is a session key for this realm, strip
+  # off everything but the value of the cookie.
+  my $cookie_name = $auth_type->cookie_name($r);
+  my ($ses_key_cookie) = ($r->header_in("Cookie") || "") =~ /$cookie_name=([^;]+)/;
+  $ses_key_cookie = "" unless defined($ses_key_cookie);
+
+  $r->log_error("ses_key_cookie " . $ses_key_cookie) if ($debug >= 1);
+  $r->log_error("uri " . $r->uri) if ($debug >= 2);
+
+  if ($ses_key_cookie) {
+    my ($auth_user, @args) = $auth_type->authen_ses_key($r, $ses_key_cookie);
+
+    if ($auth_user and scalar @args == 0) {
+      # We have a valid session key, so we return with an OK value.
+      # Tell the rest of Apache what the authentication method and
+      # user is.
+
+      $r->connection->auth_type($auth_type);
+      $r->connection->user($auth_user);
+      $r->log_error("user authenticated as $auth_user") if $debug >= 1;
+
+      # if SessionTimeout is on, send cookie with new expires
+      if (my $expires = $r->dir_config("${auth_name}SessionTimeout")) {
+        $auth_type->send_cookie($ses_key_cookie, {expires => $expires});
+      }
+
+      return OK;
+    } elsif (scalar @args > 0 and $auth_type->can('custom_errors')) {
+      return $auth_type->custom_errors($r, $auth_user, @args);
+    } else {
+      # There was a session key set, but it's invalid for some reason. So,
+      # remove it from the client now so when the credential data is posted
+      # we act just like it's a new session starting.
+      $auth_type->remove_cookie;
+      $r->subprocess_env('AuthCookieReason', 'bad_cookie');
     }
+  } else {
+    $r->subprocess_env('AuthCookieReason', 'no_cookie');
+  }
 
-    if ($r->auth_type ne $auth_type) {
-        # This location requires authentication because we are being called,
-        # but we don't handle this AuthType.
-        $r->log_error("AuthType mismatch: $auth_type =/= " . $r->auth_type)
-            if $debug >= 3;
-        return DECLINED;
-    }
-
-    # Ok, the AuthType is $auth_type which we handle, what's the authentication
-    # realm's name?
-    my $auth_name = $r->auth_name;
-    $r->log_error("auth_name " . $auth_name) if $debug >= 2;
-    unless ($auth_name) {
-        $r->log_reason("AuthName not set, AuthType=$auth_type", $r->uri);
-        return SERVER_ERROR;
-    }
-
-    # Get the Cookie header. If there is a session key for this realm, strip
-    # off everything but the value of the cookie.
-    my $cookie_name = $auth_type->cookie_name($r);
-    my ($ses_key_cookie) =
-        ($r->header_in("Cookie") || "") =~ /$cookie_name=([^;]+)/;
-    $ses_key_cookie = "" unless defined($ses_key_cookie);
-
-    $r->log_error("ses_key_cookie " . $ses_key_cookie) if ($debug >= 1);
-    $r->log_error("uri " . $r->uri) if ($debug >= 2);
-
-    if ($ses_key_cookie) {
-        my ($auth_user, @args) =
-            $auth_type->authen_ses_key($r, $ses_key_cookie);
-
-        if (!$auth_user->is_blank and scalar @args == 0) {
-
-            # We have a valid session key, so we return with an OK value.
-            # Tell the rest of Apache what the authentication method and
-            # user is.
-
-            $r->connection->auth_type($auth_type);
-            $r->connection->user($auth_user);
-            $r->log_error("user authenticated as $auth_user") if $debug >= 1;
-
-            # if SessionTimeout is on, send cookie with new expires
-            if (my $expires = $r->dir_config("${auth_name}SessionTimeout")) {
-                $auth_type->send_cookie($ses_key_cookie,
-                    { expires => $expires });
-            }
-
-            return OK;
-        }
-        elsif (scalar @args > 0 and $auth_type->can('custom_errors')) {
-            return $auth_type->custom_errors($r, $auth_user, @args);
-        }
-        else {
-
-           # There was a session key set, but it's invalid for some reason. So,
-           # remove it from the client now so when the credential data is posted
-           # we act just like it's a new session starting.
-            $auth_type->remove_cookie;
-            $r->subprocess_env('AuthCookieReason', 'bad_cookie');
-        }
-    }
-    else {
-        $r->subprocess_env('AuthCookieReason', 'no_cookie');
-    }
-
-    # They aren't authenticated, and they tried to get a protected
-    # document.  Send them the authen form.
-    return $auth_type->login_form;
+  # They aren't authenticated, and they tried to get a protected
+  # document.  Send them the authen form.
+  return $auth_type->login_form;
 }
 
-sub login_form {
-    my $self = shift;
+sub login_form {  
+  my $self = shift;
 
-    my $r = Apache->request or die "no request";
-    my $auth_name = $r->auth_name;
+  my $r = Apache->request or die "no request";
+  my $auth_name = $r->auth_name;
 
-    $self->_convert_to_get($r) if $r->method eq 'POST';
+  my %args = $r->method eq 'POST' ? $r->content : $r->args;
 
-    # There should be a PerlSetVar directive that gives us the URI of
-    # the script to execute for the login form.
+  $self->_convert_to_get($r, \%args) if $r->method eq 'POST';
 
-    my $authen_script;
-    unless ($authen_script = $r->dir_config($auth_name . "LoginScript")) {
-        $r->log_reason("PerlSetVar '${auth_name}LoginScript' not set", $r->uri);
-        return SERVER_ERROR;
-    }
-
-    #$r->log_error("Redirecting to $authen_script");
-    my $status = $self->login_form_status($r);
-    $status = FORBIDDEN unless defined $status;
-
-    if ($status == OK) {
-        # custom_response doesn't work for OK, DONE, or DECLINED in apache 1.x
-        $r->internal_redirect($authen_script);
-    }
-    else {
-        $r->custom_response($status, $authen_script);
-    }
-
-    return $status;
-}
-
-sub login_form_status {
-    my ($self, $r) = @_;
-
-    my $ua = $r->headers_in->get('User-Agent')
-        or return FORBIDDEN;
-
-    if (Apache::AuthCookie::Util::understands_forbidden_response($ua)) {
-        return FORBIDDEN;
-    }
-    else {
-        return OK;
-    }
+  # There should be a PerlSetVar directive that gives us the URI of
+  # the script to execute for the login form.
+  
+  my $authen_script;
+  unless ($authen_script = $r->dir_config($auth_name . "LoginScript")) {
+    $r->log_reason("PerlSetVar '${auth_name}LoginScript' not set", $r->uri);
+    return SERVER_ERROR;
+  }
+  #$r->log_error("Redirecting to $authen_script");
+  $r->custom_response(FORBIDDEN, $authen_script);
+  
+  return FORBIDDEN;
 }
 
 sub satisfy_is_valid {
-    my ($auth_type, $r, $satisfy) = @_;
-    $satisfy = lc $satisfy;
+  my ($auth_type, $r, $satisfy) = @_;
+  $satisfy = lc $satisfy;
 
-    if ($satisfy eq 'any' or $satisfy eq 'all') {
-        return 1;
-    }
-    else {
-        my $auth_name = $r->auth_name;
-        $r->log_reason("PerlSetVar ${auth_name}Satisfy $satisfy invalid",
-            $r->uri);
-        return 0;
-    }
+  if ($satisfy eq 'any' or $satisfy eq 'all') {
+    return 1;
+  } else { 
+    my $auth_name = $r->auth_name;
+    $r->log_reason("PerlSetVar ${auth_name}Satisfy $satisfy invalid",$r->uri);
+    return 0;
+  }
 }
 
 sub get_satisfy {
-    my ($auth_type, $r) = @_;
+  my ($auth_type, $r) = @_;
 
-    my $auth_name = $r->auth_name;
+  my $auth_name = $r->auth_name;
 
-    return lc $r->dir_config("${auth_name}Satisfy") || 'all';
+  return lc $r->dir_config("${auth_name}Satisfy") || 'all';
 }
 
 sub authorize ($$) {
-    my ($auth_type, $r) = @_;
-    my $debug = $r->dir_config("AuthCookieDebug") || 0;
-
-    $r->log_error('authorize() for ' . $r->uri()) if ($debug >= 3);
-    return OK unless $r->is_initial_req;    #only the first internal request
-
-    if ($r->auth_type ne $auth_type) {
-        $r->log_error($auth_type . " auth type is " . $r->auth_type)
-            if ($debug >= 3);
-        return DECLINED;
+  my ($auth_type, $r) = @_;
+  my $debug = $r->dir_config("AuthCookieDebug") || 0;
+  
+  $r->log_error('authorize() for '.$r->uri()) if ($debug >= 3);
+  return OK unless $r->is_initial_req; #only the first internal request
+  
+  if ($r->auth_type ne $auth_type) {
+    $r->log_error($auth_type . " auth type is " .
+		  $r->auth_type) if ($debug >= 3);
+    return DECLINED;
+  }
+  
+  my $reqs_arr = $r->requires or return DECLINED;
+  
+  my $user = $r->connection->user;
+  unless ($user) {
+    # user is either undef or =0 which means the authentication failed
+    $r->log_reason("No user authenticated", $r->uri);
+    return FORBIDDEN;
+  }
+  
+  my $satisfy = $auth_type->get_satisfy($r);
+  return SERVER_ERROR unless $auth_type->satisfy_is_valid($r,$satisfy);
+  my $satisfy_all = $satisfy eq 'all';
+  
+  my ($forbidden);
+  foreach my $req (@$reqs_arr) {
+    my ($requirement, $args) = split /\s+/, $req->{requirement}, 2;
+    $args = '' unless defined $args;
+    $r->log_error("requirement := $requirement, $args") if $debug >= 2;
+    
+    if ( lc($requirement) eq 'valid-user' ) {
+      if ($satisfy_all) {
+        next;
+      } else {
+        return OK;
+      }
     }
 
-    my $reqs_arr = $r->requires or return DECLINED;
-
-    my $user = $r->connection->user;
-    if ($user->is_blank) {
-        # authentication failed
-        $r->log_reason("No user authenticated", $r->uri);
-        return FORBIDDEN;
+    if($requirement eq 'user') {
+      if ($args =~ m/\b$user\b/) {
+        next if $satisfy_all;
+        return OK; # satisfy any
+      }
+     
+      $forbidden = 1;
+      next;
     }
 
-    my $satisfy = $auth_type->get_satisfy($r);
-    return SERVER_ERROR unless $auth_type->satisfy_is_valid($r, $satisfy);
-    my $satisfy_all = $satisfy eq 'all';
-
-    my ($forbidden);
-    foreach my $req (@$reqs_arr) {
-        my ($requirement, $args) = split /\s+/, $req->{requirement}, 2;
-        $args = '' unless defined $args;
-        $r->log_error("requirement := $requirement, $args") if $debug >= 2;
-
-        if (lc($requirement) eq 'valid-user') {
-            if ($satisfy_all) {
-                next;
-            }
-            else {
-                return OK;
-            }
-        }
-
-        if ($requirement eq 'user') {
-            if ($args =~ m/\b$user\b/) {
-                next if $satisfy_all;
-                return OK;    # satisfy any
-            }
-
-            $forbidden = 1;
-            next;
-        }
-
-        # Call a custom method
-        my $ret_val = $auth_type->$requirement($r, $args);
-        $r->log_error("$auth_type->$requirement returned $ret_val")
-            if $debug >= 3;
-        if ($ret_val == OK) {
-            next if $satisfy_all;
-            return OK;    # satisfy any
-        }
-
-        # Nothing succeeded, deny access to this user.
-        $forbidden = 1;
+    # Call a custom method
+    my $ret_val = $auth_type->$requirement($r, $args);
+    $r->log_error("$auth_type->$requirement returned $ret_val") if $debug >= 3;
+    if ($ret_val == OK) {
+      next if $satisfy_all;
+      return OK; # satisfy any
     }
 
-    return $forbidden ? FORBIDDEN : OK;
+    # Nothing succeeded, deny access to this user.
+    $forbidden = 1;
+  }
+
+  return $forbidden ? FORBIDDEN : OK;
 }
 
 sub send_cookie {
-    my ($self, $ses_key, $cookie_args) = @_;
-    my $r = Apache->request();
+  my ($self, $ses_key, $cookie_args) = @_;
+  my $r = Apache->request();
 
-    $cookie_args = {} unless defined $cookie_args;
+  $cookie_args = {} unless defined $cookie_args;
 
-    my $cookie_name = $self->cookie_name($r);
+  my $cookie_name = $self->cookie_name($r);
 
-    my $cookie = $self->cookie_string(
-        request => $r,
-        key     => $cookie_name,
-        value   => $ses_key,
-        %$cookie_args
-    );
+  my $cookie = $self->cookie_string( request => $r,
+                                     key     => $cookie_name,
+                                     value   => $ses_key,
+                                     %$cookie_args );
 
-    # add P3P header if user has configured it.
-    $self->send_p3p($r);
+  # add P3P header if user has configured it.
+  my $auth_name = $r->auth_name;
+  if (my $p3p = $r->dir_config("${auth_name}P3P")) {
+    $r->err_header_out(P3P => $p3p);
+  }
 
-    $r->err_headers_out->add("Set-Cookie" => $cookie);
+  $r->err_headers_out->add("Set-Cookie" => $cookie);
 }
 
-# send a P3P header if it is configured as ${auth_name}P3P
-sub send_p3p {
-    my ($self, $r) = @_;
-
-    my $auth_name = $r->auth_name;
-
-    if (my $p3p = $r->dir_config("${auth_name}P3P")) {
-        $r->err_header_out(P3P => $p3p);
-    }
-}
 
 sub cookie_string {
-    my $self = shift;
+  my $self = shift;
 
-    # if passed 3 args, we have old-style call.
-    if (scalar(@_) == 3) {
-        carp "deprecated old style call to "
-            . __PACKAGE__
-            . "::cookie_string()";
-        my ($r, $key, $value) = @_;
-        return $self->cookie_string(request => $r, key => $key,
-            value => $value);
-    }
+  # if passed 3 args, we have old-style call.
+  if (scalar(@_) == 3) {
+    carp "deprecated old style call to ".__PACKAGE__."::cookie_string()";
+    my ($r, $key, $value) = @_;
+    return $self->cookie_string(request=>$r, key=>$key, value=>$value);
+  }
+  # otherwise assume named parameters.
+  my %p = @_;
+  for (qw/request key/) {    
+    croak "missing required parameter $_" unless defined $p{$_};
+  }
+  # its okay if value is undef here.
 
-    # otherwise assume named parameters.
-    my %p = @_;
-    for (qw/request key/) {
-        croak "missing required parameter $_" unless defined $p{$_};
-    }
+  my $r = $p{request};
 
-    # its okay if value is undef here.
+  $p{value} = '' unless defined $p{value};
 
-    my $r = $p{request};
+  my $string = sprintf '%s=%s', @p{'key','value'};
 
-    $p{value} = '' unless defined $p{value};
+  my $auth_name = $r->auth_name;
 
-    my $string = sprintf '%s=%s', @p{ 'key', 'value' };
+  if (my $expires = $p{expires} || $r->dir_config("${auth_name}Expires")) {
+    $expires = Apache::AuthCookie::Util::expires($expires);
+    $string .= "; expires=$expires";
+  }
 
-    my $auth_name = $r->auth_name;
+  $string .= '; path=' . ( $self->get_cookie_path($r) || '/' );
+  #$r->log_error("Attribute ${auth_name}Path not set") unless $path;
 
-    if (my $expires = $p{expires} || $r->dir_config("${auth_name}Expires")) {
-        $expires = Apache::AuthCookie::Util::expires($expires);
-        $string .= "; expires=$expires";
-    }
+  if (my $domain = $r->dir_config("${auth_name}Domain")) {
+    $string .= "; domain=$domain";
+  }
 
-    $string .= '; path=' . ($self->get_cookie_path($r) || '/');
+  if ($r->dir_config("${auth_name}Secure")) {
+    $string .= '; secure';
+  }
 
-    #$r->log_error("Attribute ${auth_name}Path not set") unless $path;
+  # HttpOnly is an MS extension.  See
+  # http://msdn.microsoft.com/workshop/author/dhtml/httponly_cookies.asp
+  if ($r->dir_config("${auth_name}HttpOnly")) {
+    $string .= '; HttpOnly';
+  }
 
-    if (my $domain = $r->dir_config("${auth_name}Domain")) {
-        $string .= "; domain=$domain";
-    }
-
-    if ($r->dir_config("${auth_name}Secure")) {
-        $string .= '; secure';
-    }
-
-    # HttpOnly is an MS extension.  See
-    # http://msdn.microsoft.com/workshop/author/dhtml/httponly_cookies.asp
-    if ($r->dir_config("${auth_name}HttpOnly")) {
-        $string .= '; HttpOnly';
-    }
-
-    return $string;
+  return $string;
 }
 
 sub key {
-    my $self = shift;
-    my $r    = Apache->request;
+  my $self = shift;
+  my $r = Apache->request;
 
-    my $allcook = ($r->header_in("Cookie") || "");
-    my $cookie_name = $self->cookie_name($r);
-    return ($allcook =~ /(?:^|\s)$cookie_name=([^;]*)/)[0];
+  my $allcook = ($r->header_in("Cookie") || "");
+  my $cookie_name = $self->cookie_name($r);
+  return ($allcook =~ /(?:^|\s)$cookie_name=([^;]*)/)[0];
 }
 
 sub get_cookie_path {
     my $self = shift;
-    my $r = shift || Apache->request;
+    my $r    = shift || Apache->request;
 
     my $auth_name = $r->auth_name;
 
@@ -544,23 +453,19 @@ sub get_cookie_path {
 
 1;
 
-=pod
+__END__
 
 =head1 NAME
 
 Apache::AuthCookie - Perl Authentication and Authorization via cookies
 
-=head1 VERSION
-
-version 3.19_01
-
 =head1 SYNOPSIS
 
 Make sure your mod_perl is at least 1.24, with StackedHandlers,
 MethodHandlers, Authen, and Authz compiled in.
 
  # In httpd.conf or .htaccess:
- PerlModule Sample::Apache::AuthCookieHandler
+ PerlModule Sample::AuthCookieHandler
  PerlSetVar WhatEverPath /
  PerlSetVar WhatEverLoginScript /login.pl
 
@@ -602,26 +507,26 @@ MethodHandlers, Authen, and Authz compiled in.
 
  # These documents require user to be logged in.
  <Location /protected>
-  AuthType Sample::Apache::AuthCookieHandler
+  AuthType Sample::AuthCookieHandler
   AuthName WhatEver
-  PerlAuthenHandler Sample::Apache::AuthCookieHandler->authenticate
-  PerlAuthzHandler Sample::Apache::AuthCookieHandler->authorize
+  PerlAuthenHandler Sample::AuthCookieHandler->authenticate
+  PerlAuthzHandler Sample::AuthCookieHandler->authorize
   require valid-user
  </Location>
 
  # These documents don't require logging in, but allow it.
  <FilesMatch "\.ok$">
-  AuthType Sample::Apache::AuthCookieHandler
+  AuthType Sample::AuthCookieHandler
   AuthName WhatEver
-  PerlFixupHandler Sample::Apache::AuthCookieHandler->recognize_user
+  PerlFixupHandler Sample::AuthCookieHandler->recognize_user
  </FilesMatch>
 
  # This is the action of the login.pl script above.
  <Files LOGIN>
-  AuthType Sample::Apache::AuthCookieHandler
+  AuthType Sample::AuthCookieHandler
   AuthName WhatEver
   SetHandler perl-script
-  PerlHandler Sample::Apache::AuthCookieHandler->login
+  PerlHandler Sample::AuthCookieHandler->login
  </Files>
 
 =head1 DESCRIPTION
@@ -705,18 +610,6 @@ command
 into your server setup file and your cookies for this AuthCookie realm will be
 named MyCustomName.  Default is AuthType_AuthName.
 
-=item 6.
-
-By default users must satisfy ALL of the C<require> directives.  If you
-want authentication to succeed if ANY C<require> directives are met, use the
-C<Satisfy> directive.  For instance, if your AuthName is C<WhatEver>, you can
-put the command
-
- PerlSetVar WhatEverSatisfy Any
-
-into your server startup file and authentication for this realm will succeed if
-ANY of the C<require> directives are met.
-
 =back
 
 This is the flow of the authentication handler, less the details of the
@@ -725,7 +618,8 @@ the user's credentials in the Location field. They don't really change
 AuthCookie's model, but they do add another round-trip request to the
 client.
 
-=for html <PRE>
+=for html
+<PRE>
 
  (-----------------------)     +---------------------------------+
  ( Request a protected   )     | AuthCookie sets custom error    |
@@ -792,7 +686,8 @@ client.
     create must be able to determine if this session_key is valid and
     map it back to the originally authenticated user ID.
 
-=for html </PRE>
+=for html
+</PRE>
 
 =head1 METHODS
 
@@ -821,6 +716,12 @@ called as C<$r-E<gt>species($r, $args)>, where C<$args> is everything
 on your C<require> line after the word C<species>.  The method should
 return OK on success and FORBIDDEN on failure.
 
+Currently users must satisfy ALL of the C<require> directives.  I have
+heard that other Apache modules let the user satisfy ANY of the
+C<require> directives.  I don't know which is correct, I haven't found
+any Apache docs on the matter.  If you need one behavior or the other,
+be careful.  I may change it if I discover that ANY is correct.
+
 =item * authen_cred()
 
 You must define this method yourself in your subclass of
@@ -910,17 +811,6 @@ specified with the C<PerlSetVar WhatEverLoginScript> configuration
 directive. You can overwrite this method to provide your own
 mechanism.
 
-=item * login_form_status($r)
-
-This method returns the HTTP status code that will be returned with the login
-form response.  The default behaviour is to return FORBIDDEN, except for some
-known browsers which ignore HTML content for FORBIDDEN responses (e.g.:
-SymbianOS).  You can override this method to return custom codes.
-
-Note that FORBIDDEN is the most correct code to return as the given request was
-not authorized to view the requested page.  You should only change this if
-FORBIDDEN does not work.
-
 =item * logout()
 
 This is simply a convenience method that unsets the session key for
@@ -946,17 +836,31 @@ handy inside a method that implements a C<require> directive check
 (like the C<species> method discussed above) if you put any extra
 information like clearances or whatever into the session key.
 
-=item * untaint_destination($self, $uri)
-
-This method returns a modified version of the destination parameter
-before embedding it into the response header. Per default it escapes
-CR, LF and TAB characters of the uri to avoid certain types of
-security attacks. You can override it to more limit the allowed
-destinations, e.g., only allow relative uris, only special hosts or
-only limited set of characters.
-
 =back
 
+=head1 UPGRADING FROM VERSION 1.4
+
+There are a few interface changes that you need to be aware of
+when migrating from version 1.x to 2.x.  First, the authen() and
+authz() methods are now deprecated, replaced by the new authenticate()
+and authorize() methods.  The old methods will go away in a couple
+versions, but are maintained intact in this version to ease the task
+of upgrading.  The use of these methods is essentially the same, though.
+
+Second, when you change to the new method names (see previous
+paragraph), you must change the action of your login forms to the
+location /LOGIN (or whatever URL will call your module's login()
+method).  You may also want to change their METHOD to POST instead of
+GET, since that's much safer and nicer to look at (but you can leave
+it as GET if you bloody well want to, for some god-unknown reason).
+
+Third, you must change your login forms (see L<THE LOGIN SCRIPT>
+below) to indicate how requests should be redirected after a
+successful login.
+
+Fourth, you might want to take advantage of the new C<logout()>
+method, though you certainly don't have to.
+
 =head1 EXAMPLE
 
 For an example of how to use Apache::AuthCookie, you may want to check
@@ -1039,7 +943,7 @@ method can set arbitrary entries in C<$r-E<gt>subprocess_env>.
 
 If you want to let users log themselves out (something that can't be
 done using Basic Auth), you need to create a logout script.  For an
-example, see t/htdocs/docs/logout.pl.  Logout scripts may want to take
+example, see t/eg/logout.pl.  Logout scripts may want to take
 advantage of AuthCookie's C<logout()> method, which will set the
 proper cookie headers in order to clear the user's cookie.  This
 usually looks like C<$r-E<gt>auth_type-E<gt>logout($r);>.
@@ -1064,12 +968,22 @@ Now C<authen_ses_key()> looks up the C<$ses_key> passed to it and
 returns the saved login.  I use Oracle to store the session key and
 retrieve it later, see the ToDo section below for some other ideas.
 
+=head1 KNOWN LIMITATIONS
+
+If the first unauthenticated request is a POST, it will be changed to
+a GET after the user fills out the login forms, and POSTed data will
+be lost.
+
 =head2 TO DO
 
 =over 4
 
 =item *
 
+There ought to be a way to solve the POST problem in the LIMITATIONS
+section.  It involves being able to re-insert the POSTed content into
+the request stream after the user authenticates.
+
 It might be nice if the logout method could accept some parameters
 that could make it easy to redirect the user to another URI, or
 whatever.  I'd have to think about the options needed before I
@@ -1077,40 +991,29 @@ implement anything, though.
 
 =back
 
-=head1 HISTORY
+=head1 CVS REVISION
+
+$Id: AuthCookie.pm,v 1.3 2005/04/18 05:23:34 mschout Exp $
+
+=head1 AUTHOR
+
+Michael Schout <mschout@gkg.net>
 
 Originally written by Eric Bartley <bartley@purdue.edu>
 
 versions 2.x were written by Ken Williams <ken@forum.swarthmore.edu>
 
+=head1 COPYRIGHT
+
+Copyright (c) 2000 Ken Williams. All rights reserved.
+
+This program is free software; you can redistribute it and/or modify it under
+the same terms as Perl itself.
+
 =head1 SEE ALSO
 
 L<perl(1)>, L<mod_perl(1)>, L<Apache(1)>.
 
-=head1 SOURCE
-
-The development version is on github at L<http://github.com/mschout/apache-authcookie>
-and may be cloned from L<git://github.com/mschout/apache-authcookie.git>
-
-=head1 BUGS
-
-Please report any bugs or feature requests to bug-apache-authcookie@rt.cpan.org or through the web interface at:
- http://rt.cpan.org/Public/Dist/Display.html?Name=Apache-AuthCookie
-
-=head1 AUTHOR
-
-Michael Schout <mschout@cpan.org>
-
-=head1 COPYRIGHT AND LICENSE
-
-This software is copyright (c) 2000 by Ken Williams.
-
-This is free software; you can redistribute it and/or modify it under
-the same terms as the Perl 5 programming language system itself.
-
 =cut
 
-__END__
-
-
 # vim: sw=2 ts=2 ai et
