@@ -1,42 +1,36 @@
 package Apache2::AuthCookie;
-{
-  $Apache2::AuthCookie::VERSION = '3.19_01';
-}
-
-# ABSTRACT: Perl Authentication and Authorization via cookies
 
 use strict;
 
 use Carp;
-use CGI '3.12';
-use mod_perl2 '1.99022';
+use mod_perl2 qw(1.9922 StackedHandlers MethodHandlers Authen Authz);
 
 use Apache::AuthCookie::Util;
-use Apache2::AuthCookie::Params;
 use Apache2::RequestRec;
 use Apache2::RequestUtil;
 use Apache2::Log;
 use Apache2::Access;
 use Apache2::Response;
 use Apache2::Util;
-use Apache::AuthCookie::Autobox;
+use Apache2::URI;
 use APR::Table;
-use Apache2::Const qw(:common M_GET HTTP_FORBIDDEN HTTP_MOVED_TEMPORARILY HTTP_OK);
+use Apache2::Const qw(:common M_GET HTTP_FORBIDDEN HTTP_MOVED_TEMPORARILY);
+use vars qw($VERSION);
+
+# $Id: AuthCookie.pm,v 1.5 2005/04/18 05:23:34 mschout Exp $
+$VERSION = '3.07';
 
 sub recognize_user {
     my ($self, $r) = @_;
 
-    # only check if user is not already set
-    return DECLINED unless $r->user->is_blank;
-
     my $debug = $r->dir_config("AuthCookieDebug") || 0;
 
     my $auth_type = $r->auth_type;
     my $auth_name = $r->auth_name;
 
-    return DECLINED if $auth_type->is_blank or $auth_name->is_blank;
+    return DECLINED unless $auth_type and $auth_name;
 
-    return DECLINED if $r->headers_in->get('Cookie')->is_blank;
+    return DECLINED unless $r->headers_in->get('Cookie');
 
     my $cookie = $self->key($r);
     my $cookie_name = $self->cookie_name($r);
@@ -44,11 +38,11 @@ sub recognize_user {
     $r->server->log_error("cookie $cookie_name is $cookie")
         if $debug >= 2;
 
-    return DECLINED if $cookie->is_blank;
+    return DECLINED unless $cookie;
 
     my ($user,@args) = $auth_type->authen_ses_key($r, $cookie);
 
-    if (!$user->is_blank and scalar @args == 0) {
+    if ($user and scalar @args == 0) {
         $r->server->log_error("user is $user") if $debug >= 2;
 
         # send cookie with update expires timestamp if session timeout is on
@@ -62,7 +56,7 @@ sub recognize_user {
         return $auth_type->custom_errors($r, $user, @args);
     }
 
-    return $user->is_blank ? DECLINED : OK;
+    return OK;
 }
 
 sub cookie_name {
@@ -79,6 +73,7 @@ sub cookie_name {
 
 sub handle_cache {
     my ($self, $r) = @_;
+    _check_request_rec(@_);
 
     my $auth_name = $r->auth_name;
 
@@ -92,6 +87,7 @@ sub handle_cache {
 
 sub remove_cookie {
     my ($self, $r) = @_;
+    _check_request_rec(@_);
 
     my $cookie_name = $self->cookie_name($r);
 
@@ -105,12 +101,14 @@ sub remove_cookie {
     );
 
     $r->err_headers_out->add("Set-Cookie" => "$str");
-    $r->server->log_error("removed cookie $cookie_name") if $debug >= 2;
+    $r->server->log_error("removed_cookie ".
+                          $r->err_headers_out->get("Set-Cookie"))
+        if $debug >= 2;
 }
 
 # convert current request to GET
 sub _convert_to_get {
-    my ($self, $r) = @_;
+    my ($self, $r, $args) = @_;
 
     return unless $r->method eq 'POST';
 
@@ -118,18 +116,14 @@ sub _convert_to_get {
 
     $r->server->log_error("Converting POST -> GET") if $debug >= 2;
 
-    my $args = $self->params($r);
-
-    my @pairs = ();
-
-    for my $name ($args->param) {
+    my @pairs =();
+    while (my ($name, $value) = each %$args) {
         # we dont want to copy login data, only extra data
         next if $name eq 'destination'
              or $name =~ /^credential_\d+$/;
 
-        for my $v ($args->param($name)) {
-            push @pairs, escape_uri($r, $name) . '=' . escape_uri($r, $v);
-        }
+        $value = '' unless defined $value;
+        push @pairs, escape_uri($r, $name) . '=' . escape_uri($r, $value);
     }
 
     $r->args(join '&', @pairs) if scalar(@pairs) > 0;
@@ -145,10 +139,38 @@ sub escape_uri {
 }
 
 # get GET or POST data and return hash containing the data.
-sub params {
+sub _get_form_data {
     my ($self, $r) = @_;
 
-    return Apache2::AuthCookie::Params->new($r);
+    my $data = '';
+
+    if ($r->method eq 'POST') {
+        # $r->content doesnt exist anymore. This is the Apache::compat
+        # version.  Maybe there is more efficient way to do this...
+        $r->setup_client_block;
+
+        return unless $r->should_client_block;
+
+        my $buf;
+        while (my $read_len = $r->get_client_block($buf, 8192)) {
+            if ($read_len == -1) {
+                die "get_client_block() error";
+            }
+            $data .= $buf;
+        }
+    }
+    else {
+        $data = $r->args;
+    }
+
+    my %args = ();
+
+    if (defined $data) {
+        %args = map { Apache2::URI::unescape_url($_) }
+                split /[=&;]/, $data;
+    }
+
+    return %args;
 }
 
 sub login {
@@ -157,15 +179,14 @@ sub login {
     my $debug = $r->dir_config("AuthCookieDebug") || 0;
 
     my $auth_type = $r->auth_type;
-    my $auth_name = $r->auth_name;
 
-    my $params = $self->params($r);
+    my %args = $self->_get_form_data($r);
 
     if ($r->method eq 'POST') {
-        $self->_convert_to_get($r);
+        $self->_convert_to_get($r, \%args);
     }
 
-    unless (defined $params->param('destination')) {
+    unless (exists $args{'destination'}) {
         $r->server->log_error("No key 'destination' found in form data");
         $r->subprocess_env('AuthCookieReason', 'no_cookie');
         return $auth_type->login_form($r);
@@ -173,22 +194,18 @@ sub login {
 
     # Get the credentials from the data posted by the client
     my @credentials;
-    for (my $i = 0; defined $params->param("credential_$i"); $i++) {
-        my $key = "credential_$i";
-        my $val = $params->param($key);
-        $r->server->log_error("$key $val") if $debug >= 2;
-        push @credentials, $val;
+    while (exists $args{"credential_" . scalar(@credentials)}) {
+        my $key = 'credential_'.scalar(@credentials);
+        $r->server->log_error("$key $args{$key}") if $debug >= 2;
+        push @credentials, $args{$key};
     }
 
-    # save creds in pnotes so login form script can use them if it wants to
-    $r->pnotes("${auth_name}Creds", \@credentials);
-
     # Exchange the credentials for a session key.
     my $ses_key = $self->authen_cred($r, @credentials);
     unless ($ses_key) {
         $r->server->log_error("Bad credentials") if $debug >= 2;
         $r->subprocess_env('AuthCookieReason', 'bad_credentials');
-        $r->uri($params->param('destination'));
+        $r->uri($args{'destination'});
         return $auth_type->login_form($r);
     }
 
@@ -202,21 +219,14 @@ sub login {
     $self->handle_cache($r);
 
     if ($debug >= 2) {
-        $r->server->log_error("redirect to ", $params->param('destination'));
+        $r->server->log_error("redirect to $args{destination}");
     }
 
-    $r->headers_out->set(
-        "Location" => $self->untaint_destination($params->param('destination')));
+    $r->headers_out->set("Location" => $args{'destination'});
 
     return HTTP_MOVED_TEMPORARILY;
 }
 
-sub untaint_destination {
-    my ($self, $dest) = @_;
-
-    return Apache::AuthCookie::Util::escape_destination($dest);
-}
-
 sub logout {
     my ($self,$r) = @_;
 
@@ -235,10 +245,7 @@ sub authenticate {
     $r->server->log_error("auth_type " . $auth_type) if ($debug >= 3);
 
     unless ($r->is_initial_req) {
-        if (defined $r->prev) {
-            # we are in a subrequest.  Just copy user from previous request.
-            $r->user( $r->prev->user );
-        }
+        # only authenticate the first internal redirect
         return OK;
     }
 
@@ -272,7 +279,7 @@ sub authenticate {
     if ($ses_key_cookie) {
         my ($auth_user, @args) = $auth_type->authen_ses_key($r, $ses_key_cookie);
 
-        if (!$auth_user->is_blank and scalar @args == 0) {
+        if ($auth_user and scalar @args == 0) {
             # We have a valid session key, so we return with an OK value.
             # Tell the rest of Apache what the authentication method and
             # user is.
@@ -312,11 +319,14 @@ sub authenticate {
 
 sub login_form {
     my ($self, $r) = @_;
+    _check_request_rec(@_);
 
     my $auth_name = $r->auth_name;
 
+    my %args = $self->_get_form_data($r);
+
     if ($r->method eq 'POST') {
-        $self->_convert_to_get($r);
+        $self->_convert_to_get($r, \%args);
     }
 
     # There should be a PerlSetVar directive that gives us the URI of
@@ -328,26 +338,9 @@ sub login_form {
         return SERVER_ERROR;
     }
 
-    my $status = $self->login_form_status($r);
-    $status = HTTP_FORBIDDEN unless defined $status;
+    $r->custom_response(HTTP_FORBIDDEN, $authen_script);
 
-    $r->custom_response($status, $authen_script);
-
-    return $status;
-}
-
-sub login_form_status {
-    my ($self, $r) = @_;
-
-    my $ua = $r->headers_in->get('User-Agent')
-        or return HTTP_FORBIDDEN;
-
-    if (Apache::AuthCookie::Util::understands_forbidden_response($ua)) {
-        return HTTP_FORBIDDEN;
-    }
-    else {
-        return HTTP_OK;
-    }
+    return HTTP_FORBIDDEN;
 }
 
 sub satisfy_is_valid {
@@ -392,10 +385,9 @@ sub authorize {
 
     my $user = $r->user;
 
-    $r->server->log_error("authorize user=$user type=$auth_type") if $debug >=3;
-
-    if ($user->is_blank) {
-        # the authentication failed
+    $r->server->log_error("authorize user=$user type=$auth_type");
+    unless ($user) {
+        # user is either undef or =0 which means the authentication failed
         $r->server->log_error("No user authenticated", $r->uri);
         return HTTP_FORBIDDEN;
     }
@@ -446,6 +438,7 @@ sub authorize {
 
 sub send_cookie {
     my ($self, $r, $ses_key, $cookie_args) = @_;
+    _check_request_rec(@_);
 
     $cookie_args = {} unless defined $cookie_args;
 
@@ -458,21 +451,16 @@ sub send_cookie {
         %$cookie_args
     );
 
-    $self->send_p3p($r);
+    my $auth_name = $r->auth_name;
+
+    # add a P3P header if user has configured it.
+    if (my $p3p = $r->dir_config("${auth_name}P3P")) {
+        $r->err_headers_out->set(P3P => $p3p);
+    }
 
     $r->err_headers_out->add("Set-Cookie" => $cookie);
 }
 
-# send P3P header if configured with ${auth_name}P3P
-sub send_p3p {
-    my ($self, $r) = @_;
-
-    my $auth_name = $r->auth_name;
-
-    if (my $p3p = $r->dir_config("${auth_name}P3P")) {
-        $r->err_headers_out->set(P3P => $p3p);
-    }
-}
 
 # cookie_string takes named parameters:
 #    request
@@ -522,6 +510,7 @@ sub cookie_string {
 
 sub key {
     my ($self, $r) = @_;
+    _check_request_rec(@_);
 
     my $cookie_name = $self->cookie_name($r);
 
@@ -538,25 +527,39 @@ sub get_cookie_path {
     return $r->dir_config("${auth_name}Path");
 }
 
+# this just checks that the second arg is an Apache::RequestRec object.
+# if not, an exception is thrown.
+sub _check_request_rec {
+    my ($self, $r) = @_;
+
+    my $func = (caller)[3];
+
+    unless (defined $r) {
+        croak "missing Apache2::RequestRec argument to $func.",
+              "module not ported to mod_perl v2?";
+    }
+
+    unless (ref $r and $r->isa('Apache2::RequestRec')) {
+        croak "'$r' is not an Apache2::RequestRec object.",
+              "module not ported to mod_perl v2?";
+    }
+}
+
 1;
 
-=pod
+__END__
 
 =head1 NAME
 
 Apache2::AuthCookie - Perl Authentication and Authorization via cookies
 
-=head1 VERSION
-
-version 3.19_01
-
 =head1 SYNOPSIS
 
 Make sure your mod_perl is at least 1.24, with StackedHandlers,
 MethodHandlers, Authen, and Authz compiled in.
 
  # In httpd.conf or .htaccess:
- PerlModule Sample::Apache2::AuthCookieHandler
+ PerlModule Sample::AuthCookieHandler
  PerlSetVar WhatEverPath /
  PerlSetVar WhatEverLoginScript /login.pl
 
@@ -598,34 +601,38 @@ MethodHandlers, Authen, and Authz compiled in.
 
  # These documents require user to be logged in.
  <Location /protected>
-  AuthType Sample::Apache2::AuthCookieHandler
+  AuthType Sample::AuthCookieHandler
   AuthName WhatEver
-  PerlAuthenHandler Sample::Apache2::AuthCookieHandler->authenticate
-  PerlAuthzHandler Sample::Apache2::AuthCookieHandler->authorize
+  PerlAuthenHandler Sample::AuthCookieHandler->authenticate
+  PerlAuthzHandler Sample::AuthCookieHandler->authorize
   require valid-user
  </Location>
 
  # These documents don't require logging in, but allow it.
  <FilesMatch "\.ok$">
-  AuthType Sample::Apache2::AuthCookieHandler
+  AuthType Sample::AuthCookieHandler
   AuthName WhatEver
-  PerlFixupHandler Sample::Apache2::AuthCookieHandler->recognize_user
+  PerlFixupHandler Sample::AuthCookieHandler->recognize_user
  </FilesMatch>
 
  # This is the action of the login.pl script above.
  <Files LOGIN>
-  AuthType Sample::Apache2::AuthCookieHandler
+  AuthType Sample::AuthCookieHandler
   AuthName WhatEver
   SetHandler perl-script
-  PerlResponseHandler Sample::Apache2::AuthCookieHandler->login
+  PerlHandler Sample::AuthCookieHandler->login
  </Files>
 
 =head1 DESCRIPTION
 
-This module is for mod_perl version 2.  If you are running mod_perl version 1,
-you should be using B<Apache::AuthCookie> instead.
+WARNING! This is an early version of B<Apache::AuthCookie> for mod_perl version
+2.  If you are running under mod_perl version 2, this module should be
+considered of beta quality.  If you are a subclass author and would like to see
+additional changes (besides those listed in README.modperl2) to the interface
+provided by this module under mod_perl version 2, please contact me at
+mschout@gkg.net.  The interface for mod_perl version 1 has not changed.
 
-B<Apache2::AuthCookie> allows you to intercept a user's first
+B<Apache::AuthCookie> allows you to intercept a user's first
 unauthenticated access to a protected document. The user will be
 presented with a custom form where they can enter authentication
 credentials. The credentials are posted to the server where AuthCookie
@@ -703,18 +710,6 @@ command
 into your server setup file and your cookies for this AuthCookie realm will be
 named MyCustomName.  Default is AuthType_AuthName.
 
-=item 6.
-
-By default users must satisfy ALL of the C<require> directives.  If you
-want authentication to succeed if ANY C<require> directives are met, use the
-C<Satisfy> directive.  For instance, if your AuthName is C<WhatEver>, you can
-put the command
-
- PerlSetVar WhatEverSatisfy Any
-
-into your server startup file and authentication for this realm will succeed if
-ANY of the C<require> directives are met.
-
 =back
 
 This is the flow of the authentication handler, less the details of the
@@ -723,7 +718,8 @@ displaying the user's credentials in the Location field. They don't really
 change AuthCookie's model, but they do add another round-trip request to the
 client.
 
-=for html <PRE>
+=for html
+<PRE>
 
  (-----------------------)     +---------------------------------+
  ( Request a protected   )     | AuthCookie sets custom error    |
@@ -790,11 +786,12 @@ client.
     create must be able to determine if this session_key is valid and
     map it back to the originally authenticated user ID.
 
-=for html </PRE>
+=for html
+</PRE>
 
 =head1 METHODS
 
-C<Apache2::AuthCookie> has several methods you should know about.  Here
+C<Apache::AuthCookie> has several methods you should know about.  Here
 is the documentation for each. =)
 
 =over 4
@@ -819,10 +816,16 @@ called as C<$r-E<gt>species($r, $args)>, where C<$args> is everything
 on your C<require> line after the word C<species>.  The method should
 return OK on success and HTTP_FORBIDDEN on failure.
 
+Currently users must satisfy ALL of the C<require> directives.  I have
+heard that other Apache modules let the user satisfy ANY of the
+C<require> directives.  I don't know which is correct, I haven't found
+any Apache docs on the matter.  If you need one behavior or the other,
+be careful.  I may change it if I discover that ANY is correct.
+
 =item * authen_cred()
 
 You must define this method yourself in your subclass of
-C<Apache2::AuthCookie>.  Its job is to create the session key that will
+C<Apache::AuthCookie>.  Its job is to create the session key that will
 be preserved in the user's cookie.  The arguments passed to it are:
 
  sub authen_cred ($$\@) {
@@ -845,7 +848,7 @@ doesn't tamper with the session key.  More info in the Eagle book.
 =item * authen_ses_key()
 
 You must define this method yourself in your subclass of
-Apache2::AuthCookie.  Its job is to look at a session key and determine
+Apache::AuthCookie.  Its job is to look at a session key and determine
 whether it is valid.  If so, it returns the username of the
 authenticated user.
 
@@ -888,7 +891,7 @@ again').
     return($CODE);
   }
           
-  where CODE is a valid code from Apache2::Const
+  where CODE is a valid code from Apache::Constants
 
 =item * login()
 
@@ -908,17 +911,6 @@ specified with the C<PerlSetVar WhatEverLoginScript> configuration
 directive. You can overwrite this method to provide your own
 mechanism.
 
-=item * login_form_status($r)
-
-This method returns the HTTP status code that will be returned with the login
-form response.  The default behaviour is to return HTTP_FORBIDDEN, except for
-some known browsers which ignore HTML content for HTTP_FORBIDDEN responses
-(e.g.: SymbianOS).  You can override this method to return custom codes.
-
-Note that HTTP_FORBIDDEN is the most correct code to return as the given
-request was not authorized to view the requested page.  You should only change
-this if HTTP_FORBIDDEN does not work.
-
 =item * logout()
 
 This is simply a convenience method that unsets the session key for
@@ -944,20 +936,11 @@ handy inside a method that implements a C<require> directive check
 (like the C<species> method discussed above) if you put any extra
 information like clearances or whatever into the session key.
 
-=item * untaint_destination($self, $uri)
-
-This method returns a modified version of the destination parameter
-before embedding it into the response header. Per default it escapes
-CR, LF and TAB characters of the uri to avoid certain types of
-security attacks. You can override it to more limit the allowed
-destinations, e.g., only allow relative uris, only special hosts or
-only limited set of characters.
-
 =back
 
 =head1 EXAMPLE
 
-For an example of how to use Apache2::AuthCookie, you may want to check
+For an example of how to use Apache::AuthCookie, you may want to check
 out the test suite, which runs AuthCookie through a few of its paces.
 The documents are located in t/eg/, and you may want to peruse
 t/real.t to see the generated httpd.conf file (at the bottom of
@@ -1062,12 +1045,22 @@ Now C<authen_ses_key()> looks up the C<$ses_key> passed to it and
 returns the saved login.  I use Oracle to store the session key and
 retrieve it later, see the ToDo section below for some other ideas.
 
+=head1 KNOWN LIMITATIONS
+
+If the first unauthenticated request is a POST, it will be changed to
+a GET after the user fills out the login forms, and POSTed data will
+be lost.
+
 =head2 TO DO
 
 =over 4
 
 =item *
 
+There ought to be a way to solve the POST problem in the LIMITATIONS
+section.  It involves being able to re-insert the POSTed content into
+the request stream after the user authenticates.
+
 It might be nice if the logout method could accept some parameters
 that could make it easy to redirect the user to another URI, or
 whatever.  I'd have to think about the options needed before I
@@ -1075,7 +1068,13 @@ implement anything, though.
 
 =back
 
-=head1 HISTORY
+=head1 CVS REVISION
+
+$Id: AuthCookie.pm,v 1.5 2005/04/18 05:23:34 mschout Exp $
+
+=head1 AUTHOR
+
+Michael Schout <mschout@gkg.net>
 
 Originally written by Eric Bartley <bartley@purdue.edu>
 
@@ -1092,30 +1091,6 @@ the same terms as Perl itself.
 
 L<perl(1)>, L<mod_perl(1)>, L<Apache(1)>.
 
-=head1 SOURCE
-
-The development version is on github at L<http://github.com/mschout/apache-authcookie>
-and may be cloned from L<git://github.com/mschout/apache-authcookie.git>
-
-=head1 BUGS
-
-Please report any bugs or feature requests to bug-apache-authcookie@rt.cpan.org or through the web interface at:
- http://rt.cpan.org/Public/Dist/Display.html?Name=Apache-AuthCookie
-
-=head1 AUTHOR
-
-Michael Schout <mschout@cpan.org>
-
-=head1 COPYRIGHT AND LICENSE
-
-This software is copyright (c) 2000 by Ken Williams.
-
-This is free software; you can redistribute it and/or modify it under
-the same terms as the Perl 5 programming language system itself.
-
 =cut
 
-__END__
-
-
 # vim: sw=4 ts=4 ai et
